
/*---------------------------------------------------------------------------

    Filename: apache_log_import.js
    Project:  Apache Access Log Import Extension
    (C) Copyright 2007-2008, Kirix Corporation, All Rights Reserved.

    Summary:        Engine for Apache Log Import extenion

    Developer(s):   Benjamin I. Williams
                    David Z. Williams
                    Aaron L. Williams

---------------------------------------------------------------------------*/


class ApacheLogImport
{
    // class: ApacheLogImport
    // 
    // summary: this class imports Apache access
    // logs into a database table
 
    function importFiles(files)
    {    
        // function: importFiles(files : Array) : Boolean
        //
        // summary: imports the access logs from the input files; returns 
        // true if the files import successfully and false otherwise

        // if we don't have an output name, return false
        if (m_output.length == 0 || m_output == "/")
            return false;

        // if we don't have any input files, return false
        if (!files || files.length == 0)
            return false;

        // if a database isn't specified, use the host database
        if (!m_db)
            m_db = HostApp.getDatabase();

        // find out the total file size being imported
        var total_import_size = 0;
        for (var f in files)
        {
            var file_info = new FileInfo(files[f]);        
            total_import_size += file_info.getLength();
        }

        // create a new job to add to the job queue
        m_job = HostApp.createJob();
        m_job.setTitle("Importing Apache Logs");
        m_job.setMaximum(total_import_size);
        
        m_job.start();

        // create the table structure
        m_db.execute
        ("
            DROP TABLE IF EXISTS " + m_output + ";

            CREATE TABLE " + m_output + "
            (
                HOST VARCHAR(40),
                ACCESS_TIME DATETIME,
                METHOD VARCHAR(10),
                PROTOCOL VARCHAR(10),
                STATUS VARCHAR(10),
                BYTES NUMERIC(10,0),
                URL VARCHAR(512),
                REFERRER VARCHAR(512),
                AGENT VARCHAR(256)
            );
        ");

        // import the files
        for (var f in files)
        {
            if (!processLogFile(files[f]))
                return false;
        }

        // finish the import job
        if (m_job)
        {
            m_job.finish();
            m_job.setTitle("Importing Apache Logs");
        }

        return true;
    }

    function processLogFile(filename)
    {
        // function: processLogFile(filename : String) : Boolean
        //
        // summary: parses each line in the file specified by filename,
        // and appends the results to the output table; returns true
        // if the file is processed successfully, and false otherwise

        // if we don't have an output name, return false
        if (m_output.length == 0 || m_output == "/")
        {
            cancelImport();
            return false;
        }

        // update the job's title
        if (m_job)
            m_job.setTitle("Importing Apache Logs (" + filename + ")");

        // prepare two regular expression objects to parse the common
        // and combined access log formats

        // for the Apache Common Log Format, use:
        //     new RegExp("^(\\S*)\\s.+\\[(.*)\\]\\s+\"(\\S+) (.*?) (\\S+)\" (\\S+) (\\S+)$");
        // for the Apache Combined Log Format, use:
        //     new RegExp("^(\\S*)\\s.+\\[(.*)\\]\\s+\"(\\S+) (.*?) (\\S+)\" (\\S+) (\\S+) \"(.*)\" \"(.*)\"$");

        var parse_common_exp = new RegExp("^(\\S*)\\s.+\\[(.*)\\]\\s+\"(\\S+) (.*?) (\\S+)\" (\\S+) (\\S+)$");
        var parse_combined_exp = new RegExp("^(\\S*)\\s.+\\[(.*)\\]\\s+\"(\\S+) (.*?) (\\S+)\" (\\S+) (\\S+) \"(.*)\" \"(.*)\"$");

        // try to open the text file
        var reader = File.openText(filename);
        if (!reader)
        {
            cancelImport();
            return false;
        }

        // create a table inserter to insert the parsed values into the table
        var inserter = m_db.bulkInsert(m_output, "host, access_time, method, protocol, status, bytes, url, referrer, agent");
 
        // read all the files
        var line = reader.readLine();
        while (line)
        {
            // see if the job is cancelled; if it is, return false
            if (m_job.isCancelling())
            {
                m_job.cancel();
                return false;
            }

            // note: the following multi-regular expression logic could be 
            // packed up into a tighter construction; however, by breaking
            // it out, it's clearer, and easier to expand
        
            // create a results array
            var results = null;

            // if we don't have any results, match the combined expression
            if (!results)
                results = parse_combined_exp.exec(line);

            // if we don't have any results, match the common expression
            if (!results)
                results = parse_common_exp.exec(line);

            inserter[0] = results[1];                     // host
            inserter[1] = makeSqlDate(results[2]);        // access time
            inserter[2] = results[3];                     // method
            inserter[3] = results[5];                     // protocol
            inserter[4] = results[6];                     // status
            inserter[5] = parseInt(results[7]);           // bytes
            inserter[6] = results[4];                     // url
            inserter[7] = results[8];                     // referrer
            inserter[8] = results[9];                     // agent

            inserter.insertRow();

            line = reader.readLine();
            
            // increment the job status
            m_job.setValue(m_job.getValue() + line.length);
        }

        // close the file and finish the insert
        reader.close();
        inserter.finishInsert();
        
        return true;
    }

    function cancelImport()
    {
        // function: cancelImport()
        //
        // summary: cancels the import job

        if (m_job)
        {
            m_job.cancel();
            m_job.setTitle("Importing Apache Logs");
        }
    }

    function setDatabase(db)
    {
        // function: setDatabase(database : String)
        //
        // summary: sets the database to which to import the
        // access logs from the specified input connection
        // string

        m_db = db;
    }
    
    function setOutputTable(output)
    {
        // function: setOutputTable(output : String)
        //
        // summary: sets the output table to which to import
        // the access logs from the specified input path
    
        m_output = output;
    }

    function makeSqlDate(date)
    {
        // function: makeSqlDate(date : String)
        //
        // summary: converts an input string given by date into
        // a string that can be interpreted as a date by the
        // bulk inserter

        var dt = new Date(date);
        
        if (!dt)
            return "null";

        var s =  dt.getFullYear() + "-" +
                (dt.getMonth() + 1) + "-" +
                 dt.getDate() + " " +
                 dt.getHours() + ":" +
                 dt.getMinutes() + ":" +
                 dt.getSeconds();
        
        return s;
    }

    // member variables
    var m_db;
    var m_job;
    var m_output;
}

